
-- Author: Jamie

-- This is a modularized version of Jamie's mgen v0.1:
-- https://love2d.org/forums/viewtopic.php?f=5&t=11379&start=10#p70912

local mgen = {}

mgen._waveLookupTable = {}

mgen._waveLookupTable.pure       = function (x) return math.sin(2*math.pi*x) end
mgen._waveLookupTable.triangle   = function (x) return -0.25+0.5*(x - math.floor(x)) end
mgen._waveLookupTable.sawtooth   = function (x) return math.abs(mgen._waveLookupTable.triangle(x))*2-1 end
mgen._waveLookupTable.distortion = function (x) return math.atan(math.sin(2*math.pi*x)*2)/math.pi end
mgen._waveLookupTable.clip       = function (x) return 0.5*math.min(1, math.min(-1, 3*math.sin(2*math.pi*x))) end
mgen._waveLookupTable.square     = function (x) if (x - math.floor(x) > 0.5) then return 0.2 else return -0.2 end end

mgen._defaultDecay = function (x) if (x < 0.5) then return math.min(0.5, x*10) else return math.min(5-5*x, 0.5*10^(0.5-x)) end end

function mgen.init(duration, wave)

	if not mgen._waveLookupTable[wave] then
		wave = 'pure'
	end

	mgen.notes = {}

	for k=-24,30 do
		local data = mgen._generateNoteData(k, duration, wave, mgen._defaultDecay)
		mgen.notes[k] = love.audio.newSource(data)
	end
	  
end

-- given parameters, returns a SoundData object
function mgen._generateNoteData(noteidx, duration, wave, envelope)  -- customize with closures that define waveform and envelope
	-- note idx is set up so that middle C (261.626 Hz according to Wikipedia) is 0
	-- each half-step is +1 or -1, so all the white keys are 0, 2, 4, 5, 7, 9, 11
	-- each octave is 12, so mod 12 corresponds to the note
	-- an equally tempered scale each note is 2^(1/12) higher than the previous
	-- or in other words the frequency is F*2^(k/12) where k is the note index 
	-- and F is the frequency at middle C
	local middlec = 261.626
	local freq = middlec * math.pow(2, noteidx/12)  -- frequency in Hz

	local samprate = 44100
	local nsamp = math.floor(samprate*duration)  -- number of samples

	local result = love.sound.newSoundData(nsamp, samprate, 16, 1)

	wave = mgen._waveLookupTable[wave] -- match wave id to wave function

	if not wave then
	wave = mgen._waveLookupTable['pure']
	end

	if not envelope then
	envelope = function (x) return math.min(1, x*10, 10-x*10) end
	end

	for k=1,nsamp do
		local value = envelope((k-1)/nsamp)*wave(k*freq/samprate)
		-- base = 0.8*math.sin(k*freq*2*math.pi/samprate) -- base tone
		result:setSample(k-1, value)
	end

	return result
end

return mgen